"""
Support of Opel OnStar online service. Might work also for other cars that 
are using OnStar.

"""

import asyncio
import logging


import voluptuous as vol

from homeassistant.core import callback
from homeassistant.helpers import config_validation as cv
from homeassistant.helpers.discovery import async_load_platform
from homeassistant.helpers.dispatcher import async_dispatcher_send
from homeassistant.const import (
            ATTR_BATTERY_LEVEL, CONF_PASSWORD, CONF_SCAN_INTERVAL, CONF_USERNAME, CONF_RESOURCES)

DOMAIN = 'opel_onstar'
DATA_KEY = DOMAIN

_LOGGER = logging.getLogger(__name__)
MIN_UPDATE_INTERVAL = timedelta(minutes=1)
DEFAULT_UPDATE_INTERVAL = timedelta(minutes=1)

CONF_UPDATE_INTERVAL = 'update_interval'
SIGNAL_VEHICLE_SEEN = '{}.vehicle_seen'.format(DOMAIN)

RESOURCES = {'position': ('device_tracker',),
             'lock': ('lock', 'Lock'),
             'odometer': ('sensor', 'Odometer', 'mdi:speedometer', 'km'),
             'fuel_amount': ('sensor', 'Fuel amount', 'mdi:gas-station', 'L'),
             'fuel_amount_level': (
                 'sensor', 'Fuel level', 'mdi:water-percent', '%'),
             'average_fuel_consumption': (
                 'sensor', 'Fuel consumption', 'mdi:gas-station', 'L/100 km'),
             'distance_to_empty': ('sensor', 'Range', 'mdi:ruler', 'km'),
             'washer_fluid_level': ('binary_sensor', 'Washer fluid'),
             'brake_fluid': ('binary_sensor', 'Brake Fluid'),
             'service_warning_status': ('binary_sensor', 'Service'),
             'bulb_failures': ('binary_sensor', 'Bulbs'),
             'doors': ('binary_sensor', 'Doors'),
             'windows': ('binary_sensor', 'Windows')}

CONFIG_SCHEMA = vol.Schema({
    DOMAIN: vol.Schema({
        vol.Required(CONF_USERNAME): cv.string,
        vol.Required(CONF_PASSWORD): cv.string,
        vol.Required(CONF_PIN): cv.string,
        vol.Optional(CONF_UPDATE_INTERVAL, default=DEFAULT_UPDATE_INTERVAL): (
            vol.All(cv.time_period, vol.Clamp(min=MIN_UPDATE_INTERVAL))),
        vol.Optional(CONF_NAME, default={}): vol.Schema(
            {cv.slug: cv.string}),
        vol.Optional(CONF_RESOURCES): vol.All(
            cv.ensure_list, [vol.In(RESOURCES)]),
        vol.Optional(CONF_REGION): cv.string,
        vol.Optional(CONF_SERVICE_URL): cv.string,
        vol.Optional(CONF_SCANDINAVIAN_MILES, default=False): cv.boolean,
    }),
}, extra=vol.ALLOW_EXTRA)




"""
Class for connecting to OnStar service for getting status of your car

"""
import aiohttp
import asyncio
import json
from collections import namedtuple


class OnStar:
    """Base class for connection to OnStar service"""

    def __init__(self,username, password, pin, loop, dump_json = False):
        """Initiate connection and fetch login data"""
        self._username = username
        self._password = password
        self._pin = pin
        self._loop = loop
        self._dump_json = dump_json
        self._session = None
        self._token = None
        self._vehicle_id = None
        self._LOGIN_URL = 'https://gsp.eur.onstar.com/gspserver/services/admin/login.json'
        self._LOGINIFO_URL = 'https://gsp.eur.onstar.com/gspserver/services/admin/getLoginInfo.json'
        self._DIAGNOSTICS_URL = 'https://gsp.eur.onstar.com/gspserver/services/vehicle/getDiagnosticsReport.json'
        self._POSITION_URL = 'https://gsp.eur.onstar.com/gspserver/services/vehicle/performLocationHistoryQuery.json'
        
    
    def refresh(self):
        yield from self._login()
        yield from self._login_info()
        yield from self._diagnostics()
        yield from self._location()
        self._session.close()


    def dump_json(self, raw_string):
        if self._dump_json:
            print(json.dumps(json.loads(raw_string), sort_keys=True, indent=4, separators=(',', ': ')))


    @asyncio.coroutine
    def _login(self):
        payload = {'username': self._username, 'password': self._password, 'roleCode': 'driver', 'place': ''}

        self._session = aiohttp.ClientSession(loop=self._loop)

        response = yield from self._session.post(self._LOGIN_URL, data=payload)
        response_data = yield from response.text()
        self.dump_json(response_data)

        data = json.loads(response_data, object_hook=lambda d: namedtuple('X', d.keys())(*d.values()))
        self._login_object = data
        self._token = data.results[0].token
        self._header = {'X-GM-token': self._token}


    @asyncio.coroutine
    def _login_info(self):
        response = yield from self._session.get(self._LOGINIFO_URL, headers=self._header)
        login_info_data = yield from response.text()
        self.dump_json(login_info_data)
        self._login_info_object = json.loads(login_info_data, object_hook=lambda d: namedtuple('X', d.keys())(*d.values()))

        self._vehicle_id = self._login_info_object.results[0].vehicles[0].vehicle.vehicleId
        return self._login_info_object

    
    @asyncio.coroutine
    def _diagnostics(self):
        payload = {'vehicleId': self._vehicle_id}

        response = yield from self._session.get(self._DIAGNOSTICS_URL, params = payload, headers = self._header)
        diagnostics = yield from response.text()
        self.dump_json(diagnostics)
        diagnostics = diagnostics.replace('def','def_') #there is def field which must be renamed
        self._diagnostics_object = json.loads(diagnostics, object_hook=lambda d: namedtuple('X', d.keys())(*d.values()))
        return self._diagnostics_object


    @asyncio.coroutine
    def _location(self):
        payload = {'vehicleId': self._vehicle_id}
        header = {'X-GM-token': self._token, 'X-GM-pincode': self._pin}
        
        response = yield from self._session.post(self._POSITION_URL, params = payload, headers = header)
        location = yield from response.text()
        self.dump_json(location)
        self._location_object = json.loads(location, object_hook=lambda d: namedtuple('X', d.keys())(*d.values()))
        return self._location_object


    def get_login_info(self):
        return self._login_info_object


    def get_diagnostics(self):
        return self._diagnostics_object


    def get_location(self):
        return self._location_object



if  __name__ == "__main__":
    import getpass

    print("This demo will connect to: https://gsp.eur.onstar.com/\n")
    print("Before trying - ensure you have access by login in above site\n")
    print("\nProvide credentials\n")
    username = input("Username/email: ")
    password = getpass.getpass("Password: ")
    gm_pin = getpass.getpass("PIN for localization: ")

    loop = asyncio.get_event_loop()
    o = OnStar(username, password, gm_pin, loop, True)

    loop.run_until_complete(o.refresh())



import aiohttp
import asyncio
import json
from collections import namedtuple
import getpass

print("This demo will connect to: https://gsp.eur.onstar.com/\n")
print("Before trying - ensure you have access by login in above site\n")
print("\nProvide credentials\n")
username = input("Username/email: ")
password = getpass.getpass("Password: ")
gm_pin = getpass.getpass("PIN for localization: ")


def _json_object_hook(d): return namedtuple('X', d.keys())(*d.values())

def dumpJson(parsed):
    print(json.dumps(parsed, sort_keys=True, indent=4))

@asyncio.coroutine
def fetch(loop):
    payload = {'username': username, 'password': password, 'roleCode': 'driver', 'place': ''}

    session = aiohttp.ClientSession(loop=loop) 
    response = yield from session.post('https://gsp.eur.onstar.com/gspserver/services/admin/login.json', data=payload) 
    ret = yield from response.text()
        
    return json.loads(ret, object_hook=lambda d: namedtuple('X', d.keys())(*d.values())), session

@asyncio.coroutine
def getLoginInfo(token, session):
    header = {'X-GM-token': token}

    ret = yield from session.get('https://gsp.eur.onstar.com/gspserver/services/admin/getLoginInfo.json', headers=header)
    return ret

@asyncio.coroutine
def getDiagnostics(token, session, vehicleId):
    header = {'X-GM-token': token}
    payload = {'vehicleId': vehicleId}
    ret = yield from session.get('https://gsp.eur.onstar.com/gspserver/services/vehicle/getDiagnosticsReport.json', params=payload, headers = header)
    return ret

@asyncio.coroutine
def locateCar(token, pin, session, vehicleId):
    header = {'X-GM-token': token, 'X-GM-pincode': pin}
    payload = {'vehicleId': vehicleId}
    ret = yield from session.post('https://gsp.eur.onstar.com/gspserver/services/vehicle/performLocationHistoryQuery.json', data=payload, headers = header)
    return ret




@asyncio.coroutine
def main(loop):
    data,session = ((yield from fetch(loop)))
    print("Data from fetch - looking for token\n")
    print(data)
    token = data.results[0].token
    print("Found token: ")
    print(token)
    print("\n\n")
    data  = (yield from getLoginInfo(token,session))
    object = yield from data.text()
    print("Getting login info data, looking for first vehicleId:\n")
    dumpJson(json.loads(object) )
    object = json.loads(object, object_hook=lambda d: namedtuple('X', d.keys())(*d.values()))
    vehicleId = object.results[0].vehicles[0].vehicle.vehicleId
    print("Found vehicleId: ")
    print(vehicleId)
    print ("\n")
    diag = (yield from getDiagnostics(token, session, vehicleId))
    vehDiag = yield from diag.text()
    print("Getting diagnostics information:\n")
    dumpJson(json.loads(vehDiag))
    print("Getting localization:\n")
    locate = (yield from locateCar(token, gm_pin, session, vehicleId) )
    locate = yield from locate.text()
    dumpJson(json.loads(locate))
    print("Done for now")
    session.close()

loop = asyncio.get_event_loop()
loop.run_until_complete(main(loop))



from setuptools import setup

setup(name='onstar',
      description='Utility to get data from OnStar service specially from myopel.',
      version='0.1.0',
      url='https://github.com/nyxnyx/onstar',
      author='Grzegorz Szostak',
      author_email='szostak.grzegorz@gmail.com',
      license='MIT',
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Intended Audience :: Information Technology',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 3'
      ],
      packages=['onstar']
      )
